'*************************************************************************************
'V2-0 Created:01-November-2017 16:28 Script Revision-do not delete, edit or move this line
'*************************************************************************************

'VBscript to prevent Windows 10 Start Menu failing or to recover the Start Menu after failure.

'!!!NOTE: With Windows 10 V1709 (Build16299.19) and above please try disabling Visual Effect Animations in the taskbar before using this script.
' See Script guide pdf or Interact Forum for details of how to disable Visual Effects (restart required).

'This script MUST be installed in a folder that the user has WRITE access to if logging is enabled
'e.g. C:\Users\<Yourusername>\w10spk  NOT C:\Program Files\w10spk

'This script executes differently depending on the version of Windows 10 please refer to w10spk Script Guide_V2.pdf for full details.
'w10spk Script Guide_V2.pdf also details how to create the required task scheduler entry to run w10spk.vbs periodically to PREVENT loss of the Start Menu.

'***********************************************************************************************************************************************

'Command line options are:

'Recover Windows 10 Start Menu using a Windows shortcut.
' /restart Recovers the Windows 10 Start Menu when it is failing to respond and for Windows 10 Version 1703 also terminates any suspended UWP apps.
' /nomc DEPRECATED retained for backwards compatibility - performs the same action as above.

' /popup Displays the Windows 10 Start Menu after recovering, only works with /restart and /nomc. 
' Note: This does not work if script is run in a Command Prompt window

' /log Enable activity logging (only logs process terminations) to logfile w10spk_log.txt (in the same folder as script resides)

'Options used for testing/debugging
' /debug Enables a Cscript console window and outputs debug text, useful for testing.
' /debuglog Enable all debug output to be written to logfile w10spk_log.txt (creates an entry every time the script runs)

'The following options are primarily for use by script Setup.cmd which it is recommended to run prior to implementing this script.
' Please see w10spk Script Guide_V2.pdf for details on running setup.cmd and the options available for setting up and administering w10spk.vbs

' Setup.cmd runs this script to create or modify:
'EITHER:
'	FOR WINDOWS 10 VERSION 1703 AND EARLIER:
'	A Windows Task Scheduler Library entry to run the script on a defined timed schedule (default every 30 minutes).

'OR:
'	FOR WINDOWS 10 VERSION 1709 AND LATER:
'	A Windows Task Scheduler Library entry to run the script on a defined timed schedule (default every 60 minutes).

'To run the following  options directly, open a command window, and use cscript //nologo <path to script>\w10spk.vbs /x:
' Examples: cscript //nologo C:\Users\<yourusername>\w10spk\w10spk.vbs /x:21 to create a task which runs w10spk with /log option.
' cscript //nologo C:\Users\<yourusername>\w10spk\w10spk.vbs /x:32 to delete the task created above.

' /x:[integer code]
' /x:1 Test for MC present, playing and not playing using MC MCWS. Returns TestMCStatus function return as an exit code
' /x:2 Test which Windows 10 processes are eligible for termination-NO termination is executed

' /x:10 Create a desktop shortcut to restart Windows 10 Start Menu processes
' /x:11 Create a desktop shortcut to restart processes and display Windows 10 Start Menu
' /x:12 Create a desktop shortcut to run setup.cmd

' JRiver Media Center timed interval tasks
' /x:20 Create timer entry in Windows Task Scheduler Library, script runs with no command line options (default)
' /x:21 Create timer entry in Windows Task Scheduler Library, script runs with /log command line option
' /i:nn Modifies task timer interval, nn is minutes, only used with /x:20-21 above.
' /x:30 Disable w10spk\w10spk_run_<username> task
' /x:31 Enable w10spk\w10spk_run_<username> task
' /X:32 Delete w10spk\w10spk_run_<username> task


'Start of script ***********************************************************************************************************************

'Declare script scope variables and objects
Option Explicit
Dim blnConsole, blnDebug, blnDebugLog, blnRestart, blnLog, blnNokill, blnPopUp
Dim intBuild, intDef, intReturn, intI, intX
Dim strDebugText, strLog, strLogFile, strPath, strPopText, strScriptName, strUser
Dim colArgs
Dim objFile, objFso, wshShell, objSysInfo
'Read and process command line args
Set colArgs = Wscript.Arguments.named
If colArgs.Exists("nomc") Then  blnRestart = True
If colArgs.Exists("restart") Then  blnRestart = True
If colArgs.Exists("popup") Then  blnPopUp = True
If colArgs.Exists("debug") Then  blnDebug = True
If colArgs.Exists("debuglog") Then  blnDebugLog = True
If colArgs.Exists("log") Then  blnLog = True
If colArgs.Exists("console") Then  blnConsole = True
If colArgs.Exists("x") Then intX = colArgs.Item("x") Else intX = 0
'create shell object 
Set wshShell = CreateObject( "WScript.Shell" )
'get Windows build number (safer than Version in case user is running Windows Insider builds)
intBuild = wshShell.RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\CurrentBuildNumber")
'set default intervals for task repeat depending on Windows 10 version
If intBuild < 16299 then intDef = 30 Else intDef = 60
If colArgs.Exists("i") Then
	If colArgs.Item("i") > 0 then intI = colArgs.Item("i") Else intI = intDef
Else
	IntI = IntDef
End If
'set logfile name
strLogFile = "w10spk_log.txt"
'set WriteDebug filter variables
strLog = "log"
'Get Windows username
strUser = CreateObject("WScript.Network").UserName
'get script name
strScriptName = Left(Wscript.ScriptName ,Len(Wscript.ScriptName)- 4 )
'get the path to script
strPath = Left(WScript.ScriptFullName ,(Len(Wscript.ScriptFullName) - Len(Wscript.ScriptName)) - 1 )

'change directory to path folder
wshShell.CurrentDirectory = strPath

'Run setup routines if /x:n specified
If intX > 0 Then
blnDebug = True
StartCscript
StartSetupRoutines
End If
'If /debug force cscript console
StartCscript

'Check if process termination required-either MC playing or /restart specified
If blnRestart Then
	WriteDebug "INFO: Recovering Start Menu", strLog
	StopW10Process
Elseif TestMCStatus < 2 Then
	If NOT blnNokill then WriteDebug "INFO: Activating Start Menu failure prevention", strLog
	StopW10Process
Else
	WriteDebug "INFO: No process termination required", ""
End If
'If /log or /debuglog and debug text exists write it to top of log file 
If (blnLog OR blnDebugLog)  AND NOT strDebugText = "" Then
	If NOT WriteLogfile Then
		Dim strLogError
		strLogError = "Error writing log " &  strPath &  "\" &  strLogFile &  ", check write permissions on " &  strPath &  " folder!"
		If blnDebug Then 
			WriteDebug "ERROR: " & strLogError, ""
		Else
			Dim intExitButton
			intExitButton = wshShell.Popup(strLogError,10,"W10spk unable to write log file!",48)
		End If
	End If
End If
'Hold open cscript console window until user input (if called from wscript with /debug)
If blnConsole Then
	WScript.Echo "Press ENTER to continue... OR Close Cscript Window"
	WScript.StdIn.ReadLine
End If

'End of script


'Functions ******************************************************************************************************

Function TestMCStatus
	'check MC state using com object
	'function returns 3 for No MC , 2 for MC STOPPED, 1 for MC PAUSED and 0 for MC PLAYING
	Dim objGPB
	On Error Resume Next
	'get com object, if not exist then MC is not present
	Set objGPB = GetObject(,"MediaJukebox Application").GetPlayback()
	If Err.Number <> 0 then
		TestMCstatus = 3
		WriteDebug "INFO: Media Center is not running",""
		Exit Function
	End If
	'access com playback state
	If objGPB.state = 2 Then
		TestMCstatus = 0
		WriteDebug "INFO: Media Center playback status is: PLAYING", strLog
	Elseif objGPB.state = 1 Then 
		TestMCstatus = 1
		WriteDebug "INFO: Media Center playback status is: PAUSED", strLog
	Else
		TestMCstatus = 2
		WriteDebug "INFO: Media Center playback status is: STOPPED", ""
	End If
End Function

Function WriteLogfile
	'writes current debug info to top of logfile, log is constrained to 500 lines
	'returns true if log file is written without errors.
	Dim n, i, strItem, arrLogFileList (), objFso, objFile
	Const ForWriting = 2, ForReading = 1
	Set objFso = CreateObject("Scripting.FileSystemObject")
	'Add timestamp to strDebugText and make first entry of arrLogFileList array
	strDebugText = now() &  vbCrlf &  strDebugText
	ReDim Preserve arrLogFileList(0)
	arrLogFileList(0) = strDebugText
	'if log file exists read in n lines or until eof of current log into arrLogFileList
	n = 500
	If objFso.FileExists(strLogFile) Then
		Set objFile = objFso.OpenTextFile(strLogFile, ForReading)
		i = 1
		Do Until i = n OR objFile.AtEndOfStream
		ReDim Preserve arrLogFileList(i)
		arrLogFileList(i) = objFile.ReadLine
		i = i + 1
		Loop 
		objFile.Close
	End If
	'write back to logfile
	On Error Resume Next 
	Set objFile = objFso.OpenTextFile(strLogFile, ForWriting, True)
	For Each strItem in arrLogFileList
	objFile.WriteLine strItem
	Next
	objFile.Close
	If Err.Number = 0 Then
		WriteLogfile = True
	Else
		WriteDebug "ERROR: WriteLogfile error code=" &  Err.Number, ""
		WriteLogfile = False
	End If
End Function

'Subroutines ******************************************************************************************************

Sub WriteDebug (strText,strFilter)
	'usage writedebug "text to be written to log or Console", "" for unfiltered or "text","log" for filter to log
	'Script scope variable is strLog = "log"
	'all text is written to screen if /debug, written to log if /debuglog if tagged ""
	'only text tagged "log" (strLog) is written to log if /log 
	If blnDebug Then Wscript.Echo strText
	If blnDebugLog Then 
	strDebugText = strDebugText + strText + vbCrlf
	Elseif blnLog AND strFilter = "log" Then 
	strDebugText = strDebugText + strText + vbCrlf
	End If
End Sub


Sub StopW10Process
	'Stop Windows 10 processes:
	'=<Version 1703: if any specified by strProc are suspended, if /restart terminate ShellExperienceHost, Searchui if running or suspended and any suspended UWP apps
	'=>Version 1709: if ShellExperienceHost, Searchui are suspended, if /restart terminate them if running or suspended

	Dim Return, intExitCode, blnSuspended, blnTerminate, colProcessList, colThreadList
	Dim objProcess, objThread, objWMIService, strProc, strProcessOwner, strWinProc, intRetry, IntLoopCount
	On Error Resume Next
	'Set WMI object
	Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\cimv2")
	'Always terminate/check searchui first this appears to be quickest for Start Menu restart.
	'If searchui.exe is suspended but shellexperiencehost.exe is NOT something is wrong terminate shellexperiencehost anyway
	IntLoopCount = 0
	Do 'Create loop for 2 passes
		'wmi query string
		If IntLoopCount = 0 then
			strWinProc = "Name = 'SearchUI.exe' "
		Elseif intBuild < 16299 then
			strWinProc = "Name = 'SystemSettings.exe' OR Commandline like '%windowsapps%' OR (Commandline like '%systemapps%' AND Name <> 'SearchUI.exe') "
		Else
			strWinProc = "Name = 'ShellExperienceHost.exe' "
		End If
		Set colProcessList = objWMIService.ExecQuery ("SELECT * FROM Win32_Process WHERE " &  strWinProc &  " ")

		For Each objProcess in colProcessList
			'create 'false loop' to simulate 'continue'
			Do
				'Do not bother with MicrosoftEdgeCP processes - they terminate themselves when MicrosoftEdge is terminated
				If objProcess.name = "MicrosoftEdgeCP.exe" then Exit Do 'continue with For Each
				'Only check our own processes if possible, fail safe by checking all processes if process owner not returned.
				'retry 3 times to get Process Owner
				strProcessOwner = ""
				intRetry = 0
				Do While strProcessOwner = "" AND intRetry < 3
					If intRetry > 0 then wscript.sleep 5
					call objProcess.GetOwner(strProcessOwner)
					intRetry = intRetry + 1
				loop
				If strProcessOwner = strUser OR intRetry = 3 Then
					If strProcessOwner = "" then
						WriteDebug "INFO: Failed to get process owner", ""
					End If
					'if /restart specified then terminate processes SearchUI and ShellExperienceHost without checking if suspended
					'if we are on second pass and searchui was suspended then terminate shellexperiencehost
					If blnRestart AND (objProcess.name = "ShellExperienceHost.exe" OR objProcess.name = "SearchUI.exe") then
						intExitCode = objProcess.Terminate()
						If intExitCode = 0 Then
							WriteDebug "INFO: " & objProcess.name & " unconditionally terminated", strLog
							blnTerminate = True
						Else
							WriteDebug "ERROR: Process " & objProcess.name & " NOT terminated!", strLog
						End If
						If blnPopUp AND IntLoopCount = 1 then
							wscript.sleep 500
							wshShell.SendKeys "(^{ESC})",true
						End If
						If intBuild >= 16299 AND IntLoopCount = 1 then
							Exit Sub 'we are done, no need to check for suspended processes
						End If
						Exit Do 'continue with For Each
					End If
					'check all the process threads threadwaitreason, if all	threadwaitreasons are 5 then process is suspended,
					' if any threadwaitreason is found NOT to be 5 then stop checking threads as process is not suspended
					blnSuspended = True
					Set colThreadList = objWMIService.ExecQuery _
					 ("SELECT * FROM Win32_thread WHERE processhandle = " &  objProcess.handle &  " ")
					For Each objThread in colThreadList
						If objThread.threadwaitreason <> 5 Then
							If blnNokill AND blnDebug Then
								WriteDebug "Test Mode: " & objProcess.name & " running - NOT eligible for termination", ""
							Else
								WriteDebug "INFO: " & objProcess.name & " running - NOT eligible for termination", strLog
							End If
							blnSuspended = False
							Exit For
						End If
					Next
					'terminate process if it is suspended
					'if searchUI wait 2 seconds after termination, minimizes cpu load by staggering process recreation
					'set blnRestart to make sure shellexperiencehost is terminated even if running
					If blnSuspended Then
						If blnNokill AND blnDebug Then
							WriteDebug "Test Mode: " & objProcess.name & " suspended - eligible for termination", ""
						Else
							intExitCode = objProcess.Terminate()
							If intExitCode = 0 Then
								WriteDebug "INFO: " & objProcess.name & " suspended - process terminated", strLog
								blnTerminate = True
								If objProcess.name = "SearchUI.exe" then 
									blnRestart = true
									wscript.sleep 2000
								End If
							Else
								WriteDebug "ERROR: Suspended process " & objProcess.name & " NOT terminated!", strLog
							End If
						End If
					End If
				End If
			Loop While false 'end of inner 'false' loop
		Next
		IntLoopCount = IntLoopCount + 1

	Loop Until IntLoopCount = 2 'end of outer loop

	If blnNokill AND blnDebug Then
		WriteDebug "Test Mode: No processes were terminated", ""
	Else
		If NOT blnTerminate Then WriteDebug "INFO: No processes found to terminate", strLog
	End If

End Sub

Sub StartCscript
	'Restarts script in cscript mode if /debug
	Dim strArgs, strCmd, strEngine
	'set script engine variable = Wscript or Cscript
	strEngine = UCase( Right( WScript.FullName, 12 ) )
	If blnDebug Then
		If strEngine <> "\CSCRIPT.EXE" Then
			Dim i
			'Set wshShell = CreateObject( "WScript.Shell" )
			' Recreate the list of command line arguments
			strArgs = ""
			If WScript.Arguments.Count > 0 Then
				For i = 0 To WScript.Arguments.Count -1
					strArgs = strArgs & " " & WScript.Arguments(i)
				Next
			End If
			'flag to new script instance that cscript console window is in use ( if script started by wscript)
			strArgs = strArgs & " /console"
			' Create the complete command line to rerun this script in CSCRIPT
			strCmd = "CSCRIPT.EXE //NoLogo """ & WScript.ScriptFullName & """" & strArgs
			' Rerun the script in CSCRIPT
			wshShell.run strCmd
			Wscript.Quit
		End If
	End If
End Sub

Sub StartSetupRoutines
	'allow external Setup.cmd script to access MC play/idle checks and create Task Scheduler entries etc.
	
	Select Case intX
		case 1	'check MC status and return code 
				intReturn = TestMCStatus
		case 2	'check for eligible suspended Windows 10 processes, NO termination is executed
				blnNokill = True
				StopW10Process
		'create desktop shortcuts
		case 10 CreateShortcut "wscript.exe", """" & WScript.ScriptFullName & """ /restart", "Restart Start Menu", "Runs w10spk to Restart Start Menu"
		case 11 CreateShortcut "wscript.exe", """" & WScript.ScriptFullName & """ /restart /popup", "Restart & Display Start Menu", "Runs w10spk to Restart and Display Start Menu"
		case 12 CreateShortcut "cmd.exe", "/c """ & strPath & "\setup.cmd""", "w10spk Setup", "Runs w10spk Setup.cmd"
		'create Windows Task Scheduler entries with options
		case 20	WriteTaskEntry "", intI
		case 21	WriteTaskEntry " /log", intI
		'administer existing task
		case 30 WriteTaskEntry "", 1
		case 31 WriteTaskEntry "", 1
		case 32 WriteTaskEntry "", 1
	End Select
	'Hold open cscript console window until user input (if called from wscript)
	If blnConsole Then
		WScript.Echo "Press ENTER to continue... OR Close Cscript Window"
		WScript.StdIn.ReadLine
	End If
	Wscript.Quit (intReturn)
End Sub

Sub WriteTaskEntry(strMod,intInterval)
	'writes entry in Task Scheduler Library. Usage WriteTaskEntry "/qualifiers", task repeat interval (in minutes)
	Dim  strDateTime, strDomain, strSID, strTaskArgs, strTaskCmd, strTaskMod, strTaskName, strTaskXml, strTaskDesc, strTaskTriggers, strUserId
	Dim objFile, objFso, objTaskService, objTaskFolder, objTask, objWMIService, objAccount, intPriority
	
	On Error Resume Next
	'Create the TaskService object and connect
	Set objTaskService = CreateObject("Schedule.Service")
	call objTaskService.Connect()
	'Get domain and check task scheduler connection OK
	strDomain = objTaskService.ConnectedDomain
	If Err.Number <> 0 then
		WriteDebug "ERROR: Error occurred connecting to Task Scheduler Service, unable to proceed with Task Creation" & vbCrlf &  _
		"INFO: Please check Task Scheduler service is running" & vbCrlf & vbCrlf &  _
		"INFO: Please report this error via Personal Message on the JRiver forum to user terrym@tassie" & vbCrlf &  _
		"ERROR: Error number=" & Err.Number & " Description=" & Err.Description, ""
		IntReturn = 1
		Exit Sub
	End If
	'Get the Task Scheduler Library Root Folder
	Set objTaskFolder = objTaskService.GetFolder("\")
	'Task admin: enable, disable or delete existing task if /x:30 /x:31 /x:32 specified
	If intX = 30 OR intX = 31 OR intX = 32 then
		Dim strFolder, strTask, objw10spkFolder, objw10spkTask, colTasks
		'First check folder and task exist
		strFolder = "\w10spk"
		strTask = "w10spk_run_" & strUser
		Set objw10spkFolder = objTaskService.GetFolder(strFolder)
		If Err.Number = 0 then
			set objw10spkTask = objw10spkFolder.GetTask(strTask)
		else
			WriteDebug strFolder & " task folder does not exist" & vbCrlf & "Run Setup.cmd and select Option 1 to create w10spk task", ""
			Exit Sub
		End If
		If Err.Number = 0 then
			'task exists so carry out specified action (disable, enable or delete)
			Select Case IntX 
				case 30 'disable task
						objw10spkTask.enabled = false
						If Err.Number = 0 then
							WriteDebug "INFO: Task " & strTask & " is disabled", ""
							intReturn = 0
						Else
							Writedebug "ERROR: Unable to disable task- Error number=" & Err.Number & " Description=" & Err.Description, ""
							intReturn = 1
						End If
						Exit Sub
				case 31 'enable task
						objw10spkTask.enabled = true
						If Err.Number = 0 then
							WriteDebug "INFO: Task " & strTask & " is enabled", ""
							intReturn = 0
						Else
							Writedebug "ERROR: Unable to enable task- Error number=" & Err.Number & " Description=" & Err.Description, ""
							intReturn = 1
						End If
						Exit Sub
				case 32 'delete task and folder if it is empty
						objw10spkFolder.DeleteTask strTask, 0
						If Err.Number = 0 then
							WriteDebug "INFO: Deleted task " & strTask, ""
							Set colTasks = objw10spkFolder.GetTasks(0)
							If colTasks.count = 0 then
								objTaskFolder.DeleteFolder strFolder, 0
								If Err.Number = 0 then
									WriteDebug "INFO: Deleted task folder " & strFolder, ""
									intReturn = 0
								Else
									WriteDebug "ERROR: Deleted task but unable to delete folder " & strFolder, ""
									WriteDebug "Error number=" & Err.Number & " Description=" & Err.Description, ""
									intReturn = 1
								End If
							Else
								WriteDebug "INFO: w10spk task folder still contains these tasks", ""
								For Each objw10spkTask in colTasks
									Writedebug objw10spkTask.Name, ""
								Next
							End If
						Else
							Writedebug "ERROR: Unable to delete task- Error number=" & Err.Number & " Description=" & Err.Description, ""
							intReturn = 1
						End If
						Exit Sub
			End Select
		Else
			WriteDebug strTask & " task does not exist" & vbCrlf & "Run Setup.cmd and select Option 1 to create w10spk task", ""
			Exit Sub
		End If
	End If 'end of task admin: disable, enable or delete actions

	'Create task variables

	If NOT strMod = "" Then strTaskMod = " with" &  strMod & " option"
	strTaskArgs = """" &  WScript.ScriptFullName &  """" &  strMod
	'create iso date time...isn't vbscript sooo convenient.....
	strDateTime = DatePart("yyyy",now) & "-" _
	& Right("0" & DatePart("m",now), 2) & "-" _
	& Right("0" & DatePart("d",now), 2) & "T" _
	& Right("0" & DatePart("h",now), 2) & ":" _
	& Right("0" & DatePart("n",now), 2) & ":" _
	& Right("0" & DatePart("s",now), 2)

	strUserId = strDomain &  "\" &  strUser
	'get SID, if we can't then use strUserId
	Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\cimv2")
	Set objAccount = objWMIService.Get("Win32_UserAccount.Name='" & strUser & "',Domain='" & strDomain & "'")
	strSID = objAccount.SID
	If Err.Number <> 0 then
		StrSID = StrUserId
		WriteDebug "ERROR: Failed to get SID using WMI, attempting Task Creation using UserId" & strUserId & vbcrlf & vbcrlf &  _
		"INFO: Please report this error via Personal Message on the JRiver forum to user terrym@tassie" & vbCrlf &  _
		"ERROR: Error number=" & Err.Number & " Description=" & Err.Description, ""
		Err.Clear
	End If
	If intBuild < 16299 then
		strTaskDesc = "Runs " & strScriptName & " script" &  strTaskMod &  " every " &  intInterval &  " minutes, " & _ 
		"script maintains Windows 10 Start Menu and UWP Apps responsiveness if running JRiver Media Center. " & _
		"Script action is to terminate current user's suspended Windows 10 and UWP apps processes if JRiver Media Center is playing or paused."
	Else
		strTaskDesc = "Runs " & strScriptName & " script" &  strTaskMod &  " every " &  intInterval &  " minutes, " & _ 
		"script maintains Windows10 Start Menu responsiveness if running JRiver Media Center. " & _
		"Script action is to terminate current user's ShellExperienceHost and SearchUI processes if JRiver Media Center is playing or paused."
	End If
	strTaskTriggers = _
	"<LogonTrigger>" & vbcrlf &  _
		"<Repetition>" & vbcrlf &  _
			"<Interval>PT" &  intInterval &  "M</Interval>" & vbcrlf &  _
			"<StopAtDurationEnd>false</StopAtDurationEnd>" & vbcrlf &  _
		"</Repetition>" & vbcrlf &  _
		"<Enabled>true</Enabled>" & vbcrlf &  _
		"<UserId>" & strUserId &  "</UserId>" & vbcrlf &  _
		"<Delay>PT" &  intInterval &  "M</Delay>" & vbcrlf &  _
	"</LogonTrigger>" & vbcrlf &  _
	"<RegistrationTrigger>" & vbcrlf & _
		"<Repetition>" & vbcrlf &  _
			"<Interval>PT" &  intInterval &  "M</Interval>" & vbcrlf &  _
			"<StopAtDurationEnd>false</StopAtDurationEnd>" & vbcrlf &  _
		"</Repetition>" & vbcrlf &  _
		"<Enabled>true</Enabled>" & vbcrlf &  _
	"</RegistrationTrigger>"
	intPriority = 10
	strTaskName = "w10spk\w10spk_run_" &  strUser
	'create xml template using task variables
	strTaskXml = "<?xml version=""1.0"" encoding=""UTF-16""?>" & vbcrlf &  _
	"<Task version=""1.4"" xmlns=""http://schemas.microsoft.com/windows/2004/02/mit/task"">" & vbcrlf &  _
	"<RegistrationInfo>" & vbcrlf &  _
		"<Date>" & strDateTime & "</Date>" & vbcrlf &  _
		"<Author>w10spk</Author>" & vbcrlf &  _
		"<Description>" & strTaskDesc & "</Description>" & vbcrlf &  _
		"<URI>\"& strTaskName &  "</URI>" & vbcrlf &  _
	"</RegistrationInfo>" & vbcrlf &  _
	"<Triggers>" & vbcrlf &  _
		strTaskTriggers & vbcrlf &  _
	"</Triggers>" & vbcrlf &  _
	"<Principals>" & vbcrlf &  _
		"<Principal id=""Author"">" & vbcrlf &  _
			"<UserId>" &  strSID &  "</UserId>" & vbcrlf &  _
			"<LogonType>InteractiveToken</LogonType>" & vbcrlf &  _
			"<RunLevel>LeastPrivilege</RunLevel>" & vbcrlf &  _
		"</Principal>" & vbcrlf &  _
	"</Principals>" & vbcrlf &  _
	"<Settings>" & vbcrlf &  _
		"<MultipleInstancesPolicy>IgnoreNew</MultipleInstancesPolicy>" & vbcrlf &  _
		"<DisallowStartIfOnBatteries>false</DisallowStartIfOnBatteries>" & vbcrlf &  _
		"<StopIfGoingOnBatteries>true</StopIfGoingOnBatteries>" & vbcrlf &  _
		"<AllowHardTerminate>true</AllowHardTerminate>" & vbcrlf &  _
		"<StartWhenAvailable>false</StartWhenAvailable>" & vbcrlf &  _
		"<RunOnlyIfNetworkAvailable>false</RunOnlyIfNetworkAvailable>" & vbcrlf &  _
		"<IdleSettings>" & vbcrlf &  _
			"<StopOnIdleEnd>true</StopOnIdleEnd>" & vbcrlf &  _
			"<RestartOnIdle>false</RestartOnIdle>" & vbcrlf &  _
		"</IdleSettings>" & vbcrlf &  _
		"<AllowStartOnDemand>true</AllowStartOnDemand>" & vbcrlf &  _
		"<Enabled>true</Enabled>" & vbcrlf &  _
		"<Hidden>false</Hidden>" & vbcrlf &  _
		"<RunOnlyIfIdle>false</RunOnlyIfIdle>" & vbcrlf &  _
		"<DisallowStartOnRemoteAppSession>false</DisallowStartOnRemoteAppSession>" & vbcrlf &  _
		"<UseUnifiedSchedulingEngine>false</UseUnifiedSchedulingEngine>" & vbcrlf &  _
		"<WakeToRun>false</WakeToRun>" & vbcrlf &  _
		"<ExecutionTimeLimit>PT10M</ExecutionTimeLimit>" & vbcrlf &  _
		"<Priority>" & intPriority & "</Priority>" & vbcrlf &  _
	"</Settings>" & vbcrlf &  _
	"<Actions Context=""Author"">" & vbcrlf &  _
		"<Exec>" & vbcrlf &  _
			"<Command>wscript.exe</Command>" & vbcrlf &  _
			"<Arguments>" & strTaskArgs &  "</Arguments>" & vbcrlf &  _
		"</Exec>" & vbcrlf &  _
	"</Actions>" & vbcrlf &  _
	"</Task>"

	'Register task
	Const TASK_CREATE_OR_UPDATE = &H06&
	Const TASK_LOGON_NONE = 0
	Set objTask = objTaskFolder.RegisterTask(strTaskName, strTaskXml, TASK_CREATE_OR_UPDATE, strUserId, , TASK_LOGON_NONE)

	'Check for Task Registration success, if registration fails create xml template file.
	If Err.Number = 0 then
		WriteDebug "INFO: Created or modified Task Sheduler entry: " &  objTask.Name, ""
		WriteDebug "INFO: Script " & strScriptName & ".vbs" & strMod & " will be run every " &  intInterval &  " minutes from now on", ""
		intReturn = 0
	Elseif Err.Number = 70 then
		Writedebug "ERROR: Error creating or modifying Task Scheduler entry!! Permission Denied!!" & vbcrlf & vbcrlf &  _
		"WARNING: It is likely that a task created by the system tool Task Scheduler in the w10spk folder has the same name." & vbcrlf &  _
		"WARNING: w10spk cannot create or modify a task if one with same name exists that was created by Task Scheduler." & vbcrlf & _
		"INFO: Please delete task " & strTaskName & " using Computer Management>System Tools>Task Scheduler."  & vbcrlf &  _
		"INFO: Then run setup again to create a task that can be modified by w10spk." & vbcrlf & vbcrlf & _
		"INFO: If you receive this error and there is no task with the same name in the w10spk folder please report the error via Personal Message to forum user terrym@tassie", ""
		intReturn = 1
	Else
		WriteDebug "ERROR: Error occurred creating or modifying Task Scheduler entry!", ""
		WriteDebug "ERROR: Error number=" & Err.Number & " Description=" & Err.Description & vbcrlf, ""
		intReturn = 1
		Err.Clear
		WriteDebug "ERROR: w10spk is unable to create Task Scheduler task automatically!" & vbcrlf & vbcrlf &  _
		"INFO: Please report this error via Personal Message on the JRiver forum to user terrym@tassie" & vbcrlf &  _
		"INFO: You can create a task manually using Computer Management>System Tools>Task Scheduler" & vbcrlf &  _
		"INFO: In the Task Scheduler library create a folder called w10spk_ts" & vbcrlf &  _
		"INFO: Open this folder and then import the task using the xml template file created below:"  & vbcrlf &  _
		"WARNING: Please note that a task imported using Task Scheduler cannot be modified by w10spk" & vbcrlf &  _
		"WARNING: Any changes to the imported task must be done via the Task Scheduler system tool" & vbcrlf, ""
		'create xml file for task creation
		dim strXmlFile
		strXmlFile = "w10spk_ts_run_" & strUser & ".xml"
		Const ForWriting = 2
		Set objFso = CreateObject("Scripting.FileSystemObject")
		Set objFile = objFso.OpenTextFile(strXmlFile, ForWriting, True)
		objFile.Write strTaskXml
		objFile.Close
		'Handle file creation error
		If Err.Number = 0 then
			Writedebug "INFO: Created XML template " & strPath & "\" & strXmlFile & " for manual import", ""
		Else
			WriteDebug "ERROR: Unable to write " & strPath & "\" & strXmlFile & " for manual import!" & vbcrlf & vbcrlf &  _
			"WARNING: Please check you have Write access to the w10spk script folder " & strPath, ""
		End If
	End If
End Sub

Sub CreateShortcut(strTarget,strArguments,strSC,strDesc)
	dim objShortCut, strUsersDesktop
	On Error Resume Next
	'Get the current users Desktop
	StrUsersDesktop = wshShell.SpecialFolders("Desktop")
	'Create shortcut path
	Set objShortCut = wshShell.CreateShortcut(strUsersDesktop & "\" & strSC & ".lnk")
	'set target as wscript.exe
	objShortCut.TargetPath = "%SystemRoot%\system32\" & strTarget
	'add specific arguments
	objShortCut.Arguments = strArguments
	'Add specific description
	objShortCut.Description = strDesc
	'set folder to start in
	objShortCut.WorkingDirectory = strPath
	'Create the shortcut
	objShortCut.Save
	If Err.Number = 0 then
		Writedebug "Created desktop shortcut: " & strSC, ""
		intReturn = 0
	Else
		Writedebug "Error creating desktop shortcut: " & strSC, ""
		WriteDebug "Error number=" & Err.Number & "Description=" & Err.Description
		intReturn = 1
	End If
End Sub
